#lang racket
(require (lib "graphics.ss" "graphics"))

(define-namespace-anchor a)
(define ns (namespace-anchor->namespace a))

;-- Runden auf eine bestimmte Anzahl von Nachkommestellen ----------------------

(define runden
  (lambda (x anzahl)
    (/ (round (* x (expt 10 anzahl))) (expt 10 anzahl))))

;-- Stream-Sprachelemente ------------------------------------------------------

(define stream-car car)

(define stream-cdr
  (lambda (stm)
    (force (cdr stm))))

(define-syntax stream-cons
  (syntax-rules ()
    ((_ head tail) (cons head (delay tail)))))

(define stream?
  (lambda (stm)
    (and (pair? stm) (promise? (cdr stm)))))

(define stream-map
  (lambda (proz stm)
    (stream-cons (proz (stream-car stm)) (stream-map proz (stream-cdr stm)))))

(define stream-filter
  (lambda (proz stm)
    (if (proz (stream-car stm))
        (stream-cons (stream-car stm) (stream-filter proz (stream-cdr stm)))
        (stream-filter proz (stream-cdr stm)))))

(define show-stream
  (lambda (stm n)
    (if (= n 0)
        (printf "... ~n")
        (begin
          (printf "~s, " (stream-car stm))
          (show-stream (stream-cdr stm) (- n 1))))))

(define show-stream-inexact
  (lambda (stm n anzahl)
    (if (= n 0)
        (printf "... ~n")
        (begin
          (printf "~s, " (exact->inexact (runden (stream-car stm) anzahl)))
          (show-stream-inexact (stream-cdr stm) (- n 1) anzahl)))))

;-- Stream natuerlicher Zahlen --------------------------------------------------

(define natstream
  (lambda (n)
    (stream-cons n (natstream (+ n 1)))))

;-- Balkendiagramm -------------------------------------------------------------

(define grafik '())
(define grafikbreite 400)
(define grafikhoehe 300)
(define grafikrand 50)
(define color "red")
(define maxbreite (- grafikbreite (* 2 grafikrand)))
(define maxhoehe (- grafikhoehe (* 2 grafikrand)))

(define minmax
  (lambda (stm n)
    (letrec
        ([extremsuche
          (lambda (extrem stm n op)
            (cond
              [(= n 0) extrem]
              [(op extrem (stream-car stm)) (extremsuche (stream-car stm) (stream-cdr stm) (- n 1) op)]
              [else
               (extremsuche extrem (stream-cdr stm) (- n 1) op)]))])
      (cons (extremsuche 0 stm n >)
            (extremsuche 0 stm n <)))))

(define balken
  (lambda (x y breite hoehe)
    ((draw-rectangle grafik) (make-posn x y) breite hoehe color)))

(define balkendiagramm
  (lambda (stm n)
    (let* ([breite (/ maxbreite n)]
           [extremwerte (minmax stm n)]
           [min (car extremwerte)] [max (cdr extremwerte)]
           [x grafikrand] [y (+ grafikrand (* (/ max (- max min)) maxhoehe))])
      (letrec
          ([zeichnen
            (lambda (stm n)
              (if (> n 1)
                  (let ([hoehe (* (/ (stream-car stm) (- max min)) maxhoehe)])
                    (if (> (stream-car stm) 0)
                        (balken x (- y hoehe) (+ breite 1) hoehe)
                        (balken x (- y 1) (+ breite 1) (- hoehe)))
                    (set! x (+ x breite))
                    (zeichnen (stream-cdr stm) (- n 1)))
                  (printf "~s ~n" 'ok)))])
        (open-graphics)
        (set! grafik (open-viewport "Balkendiagramm" grafikbreite grafikhoehe))          
        (zeichnen stm n)))))

(define balkendiagramm-schliessen
  (lambda ()
    (close-viewport grafik)
    (close-graphics)))

;-- Beispiel-Zahlenfolge -------------------------------------------------------

(define fkt
  (lambda (n)
    (expt (+ 1 (/ 1 n)) n)))

(define en (stream-map fkt (natstream 1)))

;(show-stream-inexact en 10 4)
;(balkendiagramm en 50)

;-- Bildschirmdialog -----------------------------------------------------------

(define dialog
  (lambda ()
    (let ([vorschrift ""] [n ""] [index ""])
      (printf "Bildungsvorschrift (Lambda-Ausdruck): ") (set! vorschrift (read))
      (printf "Anzahl der Folgenglieder: ") (set! n (read))
      (printf "Startindex (0, 1, ...)  : ") (set! index (read))
      (newline)
      (balkendiagramm (stream-map (eval vorschrift ns) (natstream index)) n))))

;(dialog)


